/*
Module : MFCSensor.h
Purpose: Defines the interface for a set of C++ wrapper classes for the Windows Sensors API
Created: PJN / 05-02-2011
History: PJN / 07-02-2011 1. CMFCLocationSensorDataReport is now derived from a new "CMFCSensorDataReport" class.
                          2. Provided a number of new SensorDataReport classes including CMFCElectricalSensorDataReport,
                          CMFCEnvironmentalSensorDataReport, CMFCLightSensorDataReport, CMFCMechanicalSensorDataReport,
                          CMFCMotionSensorDataReport, CMFCOrientationSensorDataReport & CMFCScannerSensorDataReport
         PJN / 01-12-2013 1. Updated copyright details.
                          2. Updated the code to clean compile in VC 2012/VC 2013
                          3. Reworked the classes to optionally compile without MFC. By default the classes now use
                          STL for strings and arrays but if you define CBITSWRAPPERS_MFC_EXTENSIONS, the classes will
                          revert back to using the MFC CString and CArray classes.
                          4. The classes are now in a "SensorAPI" namespace and the MFC prefix for all the classes have
                          been removed.
         PJN / 13-01-2016 1. Updated copyright details.
                          2. The preprocessor define to enable MFC integration is now CMFCSENSOR_MFC_EXTENSIONS instead 
                          of CBITSWRAPPERS_MFC_EXTENSIONS which was a copy and paste bug.
                          3. Verified the code compiles cleanly in VC 2015
                          4. Added SAL annotations to all the code
                          5. CSensor::GetFriendlyName now uses ATL::CComBSTR internally
         PJN / 25-11-2017 1. Updated copyright details.
                          2. Updated the code to compile cleanly when _ATL_NO_AUTOMATIC_NAMESPACE is defined.
                          3. Replaced NULL throughout the codebase with nullptr.
         PJN / 21-10-2018 1. Updated copyright details.
                          2. Fixed a number of C++ core guidelines compiler warnings. These changes mean that the code 
                          will now only compile on VC 2017 or later.
                          3. Removed all code which supported CMFCSENSOR_MFC_EXTENSIONS code path
         PJN / 04-10-2019 1. Fixed a number of compiler warnings when the code is compiled with VS 2019 Preview
                          2. Updated copyright details.
         PJN / 22-03-2020 1. Updated copyright details.
                          2. Fixed more Clang-Tidy static code analysis warnings in the code.
         PJN / 29-04-2022 1. Updated copyright details.
                          2. Updated the code to use C++ uniform initialization for all variable declarations.

Copyright (c) 2011 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#pragma once

#ifndef __MFCSENSOR_H__
#define __MFCSENSOR_H__


//////////////////// Includes /////////////////////////////////////////////////

#ifndef __sensorsapi_h__
#pragma message("To avoid this message, please put sensorsapi.h in your pre compiled header (normally stdafx.h)")
#include <sensorsapi.h>
#endif //#ifndef __sensorsapi_h__

#ifndef _SENSORS_H_
#pragma message("To avoid this message, please put sensors.h in your pre compiled header (normally stdafx.h)")
#include <sensors.h>
#endif //#ifndef _SENSORS_H_

#ifndef __ATLCOM_H__
#pragma message("To avoid this message, please put atlcom.h in your pre compiled header (normally stdafx.h)")
#include <atlcom.h>
#endif //#ifndef __ATLCOM_H__

#ifndef _VECTOR_
#pragma message("To avoid this message, please put vector in your pre compiled header (normally stdafx.h)")
#include <vector>
#endif //#ifndef _VECTOR_

#ifndef _STRING_
#pragma message("To avoid this message, please put string in your pre compiled header (normally stdafx.h)")
#include <string>
#endif //#ifndef _STRING_

#pragma comment(lib, "sensorsapi.lib") //Automatically link to the Sensor API library


//////////////////// Classes //////////////////////////////////////////////////

namespace SensorAPI
{

//The main wrapper for the ISensorManagerEvents interface
class ATL_NO_VTABLE CSensorManagerEvents :
  public ATL::CComObjectRootEx<ATL::CComSingleThreadModel>,
  public ISensorManagerEvents
{
public:
#pragma warning(suppress: 26440)
  DECLARE_NO_REGISTRY()

#pragma warning(suppress: 26429 26440 26447 26477 26481)
  BEGIN_COM_MAP(CSensorManagerEvents) //NOLINT(modernize-avoid-c-arrays, modernize-use-noexcept)
    COM_INTERFACE_ENTRY(ISensorManagerEvents)
#pragma warning(suppress: 26485)
  END_COM_MAP()

//ISensorManagerEvents
#pragma warning(suppress: 26433 26436 26440 26812)
  STDMETHOD(OnSensorEnter)(__RPC__in_opt ISensor* /*pSensor*/, SensorState /*state*/)
  {
    return E_NOTIMPL;
  }
};

//The main wrapper for the ISensorManager interface
template <class SensorManagerEvents = CSensorManagerEvents>
class CSensorManager : public ATL::CComPtr<ISensorManager>
{
public:
//Methods
  HRESULT Initialize() noexcept
  {
    return CoCreateInstance(CLSID_SensorManager);
  }

  HRESULT SetEventSink()
  {
    ATL::CComObject<SensorManagerEvents>* pSinkEvent{nullptr};
    HRESULT hr{ATL::CComObject<SensorManagerEvents>::CreateInstance(&pSinkEvent)};
    if (SUCCEEDED(hr))
      hr = p->SetEventSink(pSinkEvent);

    return hr;
  }
};

//The main wrapper for the ISensorEvents interface
class ATL_NO_VTABLE CSensorEvents :
  public ATL::CComObjectRootEx<ATL::CComSingleThreadModel>,
  public ISensorEvents
{
public:
#pragma warning(suppress: 26440)
  DECLARE_NO_REGISTRY()

#pragma warning(suppress: 26429 26440 26447 26477 26481)
  BEGIN_COM_MAP(CSensorEvents) //NOLINT(modernize-avoid-c-arrays, modernize-use-noexcept)
    COM_INTERFACE_ENTRY(ISensorEvents)
#pragma warning(suppress: 26477 26485)
  END_COM_MAP()

//ISensorEvents
#pragma warning(suppress: 26433 26436 26440)
  STDMETHOD(OnEvent)(__RPC__in_opt ISensor* /*pSensor*/, __RPC__in REFGUID /*eventID*/, __RPC__in_opt IPortableDeviceValues* /*pEventData*/)
  {
    return E_NOTIMPL;
  }
#pragma warning(suppress: 26433 26440)
  STDMETHOD(OnDataUpdated)(__RPC__in_opt ISensor* /*pSensor*/, __RPC__in_opt ISensorDataReport* /*pNewData*/)
  {
    return E_NOTIMPL;
  }
#pragma warning(suppress: 26433 26440)
  STDMETHOD(OnLeave)(__RPC__in_opt REFSENSOR_ID /*sensorID*/)
  {
    return E_NOTIMPL;
  }
#pragma warning(suppress: 26433 26440)
  STDMETHOD(OnStateChanged)(__RPC__in_opt ISensor* /*pSensor*/, SensorState /*state*/)
  {
    return E_NOTIMPL;
  }
};

//The main wrapper for the ISensor interface
template <class SensorEvents = CSensorEvents>
class CSensor : public ATL::CComPtr<ISensor>
{
public:
//Constructors / Destructors
  CSensor(_In_ ISensor* pSensor) : CComPtr<ISensor>{pSensor}
  {
  }
  CSensor(const CSensor&) = default;
  CSensor(CSensor&&) = delete;
  CSensor() = default;
  ~CSensor() = default;

//Methods
  CSensor& operator=(const CSensor&) = default;
  CSensor& operator=(CSensor&&) = delete;
  HRESULT SetEventSink()
  {
    ATL::CComObject<SensorEvents>* pSinkEvent{nullptr};
    HRESULT hr{ATL::CComObject<SensorEvents>::CreateInstance(&pSinkEvent)};
    if (SUCCEEDED(hr))
      hr = p->SetEventSink(pSinkEvent);

    return hr;
  }

  HRESULT GetFriendlyName(_Out_ std::wstring& sFriendlyName)
  {
    ATL::CComBSTR bstrFriendlyName;
    const HRESULT hr{p->GetFriendlyName(&bstrFriendlyName)};
    if (SUCCEEDED(hr))
      sFriendlyName = bstrFriendlyName;
    return hr;
  }
};

//The main wrapper for ISensorDataReport
class CSensorDataReport : public ATL::CComPtr<ISensorDataReport>
{
public:
//Constructors / Destructors
  CSensorDataReport(_In_ ISensorDataReport* pSensorDataReport) noexcept : CComPtr<ISensorDataReport>{pSensorDataReport}
  {
  }
  CSensorDataReport(const CSensorDataReport&) = default;
  CSensorDataReport(CSensorDataReport&&) = delete;
  CSensorDataReport() = default;
  ~CSensorDataReport() = default;

//Methods
  CSensorDataReport& operator=(const CSensorDataReport&) = default;
  CSensorDataReport& operator=(CSensorDataReport&&) = delete;
  HRESULT GetValue(_In_ PROPERTYKEY propKey, _Out_ double& dValue)
  {
    PROPVARIANT var;
#pragma warning(suppress: 26486)
    PropVariantInit(&var);
    HRESULT hr{p->GetSensorValue(propKey, &var)};
    if (FAILED(hr))
      return hr;
    if (var.vt == VT_R8)
      dValue = var.dblVal;
    else if (var.vt == VT_R4) //Some sensors can return a VT_R4 instead of a VT_R8 (Lets handle this issue transparently)
      dValue = var.fltVal;
    else
      hr = E_UNEXPECTED;
    PropVariantClear(&var);

    return hr;
  }

  HRESULT GetValue(_In_ PROPERTYKEY propKey, _Out_ float& dValue)
  {
    PROPVARIANT var;
#pragma warning(suppress: 26486)
    PropVariantInit(&var);
    HRESULT hr{p->GetSensorValue(propKey, &var)};
    if (FAILED(hr))
      return hr;
    if (var.vt == VT_R4)
      dValue = var.fltVal;
    else
      hr = E_UNEXPECTED;
    PropVariantClear(&var);

    return hr;
  }

  HRESULT GetValue(_In_ PROPERTYKEY propKey, _Out_ int& nValue)
  {
    PROPVARIANT var;
#pragma warning(suppress: 26486)
    PropVariantInit(&var);
    HRESULT hr{p->GetSensorValue(propKey, &var)};
    if (FAILED(hr))
      return hr;
    if (var.vt == VT_I4)
      nValue = var.lVal;
    else
      hr = E_UNEXPECTED;
    PropVariantClear(&var);

    return hr;
  }

  HRESULT GetValue(_In_ PROPERTYKEY propKey, _Out_ bool& nValue)
  {
    PROPVARIANT var;
#pragma warning(suppress: 26486)
    PropVariantInit(&var);
    HRESULT hr{p->GetSensorValue(propKey, &var)};
    if (FAILED(hr))
      return hr;
    if (var.vt == VT_BOOL)
      nValue = (var.boolVal == VARIANT_TRUE) ? true : false;
    else
      hr = E_UNEXPECTED;
    PropVariantClear(&var);

    return hr;
  }

  HRESULT GetValue(_In_ PROPERTYKEY propKey, _Out_ ULARGE_INTEGER& nValue)
  {
    PROPVARIANT var;
#pragma warning(suppress: 26486)
    PropVariantInit(&var);
    HRESULT hr{p->GetSensorValue(propKey, &var)};
    if (FAILED(hr))
      return hr;
    if (var.vt == VT_UI8)
      nValue = var.uhVal;
    else
      hr = E_UNEXPECTED;
    PropVariantClear(&var);

    return hr;
  }

  HRESULT GetValue(_In_ PROPERTYKEY propKey, _Out_ std::vector<double>& dblArray)
  {
    PROPVARIANT var;
#pragma warning(suppress: 26486)
    PropVariantInit(&var);
    HRESULT hr{p->GetSensorValue(propKey, &var)};
    if (FAILED(hr))
      return hr;
    if ((var.vt == (VT_VECTOR | VT_UI1)) && ((var.caub.cElems % sizeof(double)) == 0))
    {
      const size_t nElements{var.caub.cElems / sizeof(double)};
      dblArray.resize(nElements);
#pragma warning(suppress: 26429 26489 26490)
      auto pElementsToCopy{reinterpret_cast<double*>(var.caub.pElems)};
      for (size_t i{0}; i<nElements; i++)
#pragma warning(suppress: 26446 26481 26489)
        dblArray[i] = pElementsToCopy[i];
    }
    else
      hr = E_UNEXPECTED;
    PropVariantClear(&var);

    return hr;
  }

  HRESULT GetValue(_In_ PROPERTYKEY propKey, _Out_ std::vector<float>& fltArray)
  {
    PROPVARIANT var;
#pragma warning(suppress: 26486)
    PropVariantInit(&var);
    HRESULT hr{p->GetSensorValue(propKey, &var)};
    if (FAILED(hr))
      return hr;
    if ((var.vt == (VT_VECTOR | VT_UI1)) && ((var.caub.cElems % sizeof(float)) == 0))
    {
      const size_t nElements{var.caub.cElems/sizeof(float)};
      fltArray.resize(nElements);
#pragma warning(suppress: 26429 26489 26490)
      auto pElementsToCopy{reinterpret_cast<float*>(var.caub.pElems)};
      for (size_t i{0}; i<nElements; i++)
#pragma warning(suppress: 26446 26481 26489)
        fltArray[i] = pElementsToCopy[i];
    }
    else
      hr = E_UNEXPECTED;
    PropVariantClear(&var);

    return hr;
  }

  HRESULT GetValue(_In_ PROPERTYKEY propKey, _Out_ std::vector<ULONG>& ulArray)
  {
    PROPVARIANT var;
#pragma warning(suppress: 26486)
    PropVariantInit(&var);
    HRESULT hr{p->GetSensorValue(propKey, &var)};
    if (FAILED(hr))
      return hr;
    if ((var.vt == (VT_VECTOR | VT_UI1)) && ((var.caub.cElems % sizeof(ULONG)) == 0))
    {
      const size_t nElements{var.caub.cElems/sizeof(ULONG)};
      ulArray.resize(nElements);
#pragma warning(suppress: 26429 26489 26490)
      auto pElementsToCopy{reinterpret_cast<ULONG*>(var.caub.pElems)};
      for (size_t i{0}; i<nElements; i++)
#pragma warning(suppress: 26446 26481 26489)
        ulArray[i] = pElementsToCopy[i];
    }
    else
      hr = E_UNEXPECTED;
    PropVariantClear(&var);

    return hr;
  }

  HRESULT GetValue(_In_ PROPERTYKEY propKey, _Out_ std::wstring& sValue)
  {
    PROPVARIANT var;
#pragma warning(suppress: 26486)
    PropVariantInit(&var);
    HRESULT hr{p->GetSensorValue(propKey, &var)};
    if (FAILED(hr))
      return hr;
    if (var.vt == VT_LPWSTR)
      sValue = var.pwszVal;
    else
      hr = E_UNEXPECTED;
    PropVariantClear(&var);

    return hr;
  }
};

//A Location sensor specific wrapper for ISensorDataReport
class CLocationSensorDataReport : public CSensorDataReport
{
public:
//Constructors / Destructors
  CLocationSensorDataReport(_In_ ISensorDataReport* pSensorDataReport) noexcept : CSensorDataReport{pSensorDataReport}
  {
  }
  CLocationSensorDataReport(const CLocationSensorDataReport&) = default;
  CLocationSensorDataReport(CLocationSensorDataReport&&) = delete;
  CLocationSensorDataReport() = default;
  ~CLocationSensorDataReport() = default;

//Methods
  CLocationSensorDataReport& operator=(const CLocationSensorDataReport&) = default;
  CLocationSensorDataReport& operator=(CLocationSensorDataReport&&) = delete;
  HRESULT GetAddress1(_Out_ std::wstring& sAddress1)
  {
    return GetValue(SENSOR_DATA_TYPE_ADDRESS1, sAddress1);
  }

  HRESULT GetAddress2(_Out_ std::wstring& sAddress2)
  {
    return GetValue(SENSOR_DATA_TYPE_ADDRESS2, sAddress2);
  }

  HRESULT GetAntennaSealevelAltitude(_Out_ double& dAltitude)
  {
    return GetValue(SENSOR_DATA_TYPE_ALTITUDE_ANTENNA_SEALEVEL_METERS, dAltitude);
  }

  HRESULT GetElipsoidAltitudeError(_Out_ double& dAltitude)
  {
    return GetValue(SENSOR_DATA_TYPE_ALTITUDE_ELLIPSOID_ERROR_METERS, dAltitude);
  }

  HRESULT GetElipsoidAltitude(double& dAltitude)
  {
    return GetValue(SENSOR_DATA_TYPE_ALTITUDE_ELLIPSOID_METERS, dAltitude);
  }

  HRESULT GetSealeveldAltitudeError(_Out_ double& dAltitude)
  {
    return GetValue(SENSOR_DATA_TYPE_ALTITUDE_SEALEVEL_ERROR_METERS, dAltitude);
  }

  HRESULT GetSealevelAltitude(_Out_ double& dAltitude)
  {
    return GetValue(SENSOR_DATA_TYPE_ALTITUDE_SEALEVEL_METERS, dAltitude);
  }

  HRESULT GetCity(_Out_ std::wstring& sCity)
  {
    return GetValue(SENSOR_DATA_TYPE_CITY, sCity);
  }

  HRESULT GetCountryOrRegion(_Out_ std::wstring& sCountryOrRegion)
  {
    return GetValue(SENSOR_DATA_TYPE_COUNTRY_REGION, sCountryOrRegion);
  }

  HRESULT GetDifferentialReferenceStationID(_Out_ int& nID)
  {
    return GetValue(SENSOR_DATA_TYPE_DIFFERENTIAL_REFERENCE_STATION_ID, nID);
  }

  HRESULT GetErrorRadius(_Out_ double& dErrorRadius)
  {
    return GetValue(SENSOR_DATA_TYPE_ERROR_RADIUS_METERS, dErrorRadius);
  }

  HRESULT GetFixQuality(_Out_ int& nFix)
  {
    return GetValue(SENSOR_DATA_TYPE_FIX_QUALITY, nFix);
  }

  HRESULT GetDataTypeFixType(_Out_ int& nFixType)
  {
    return GetValue(SENSOR_DATA_TYPE_FIX_TYPE, nFixType);
  }

  HRESULT GetGeoidalSeparation(_Out_ double& dSeparation)
  {
    return GetValue(SENSOR_DATA_TYPE_GEOIDAL_SEPARATION, dSeparation);
  }

  HRESULT GetGPSOperationMode(_Out_ int& nMode)
  {
    return GetValue(SENSOR_DATA_TYPE_GPS_OPERATION_MODE, nMode);
  }

  HRESULT GetGPSSelectionMode(_Out_ int& nMode)
  {
    return GetValue(SENSOR_DATA_TYPE_GPS_SELECTION_MODE, nMode);
  }

  HRESULT GetGPSStatus(_Out_ int& nStatus)
  {
    return GetValue(SENSOR_DATA_TYPE_GPS_STATUS, nStatus);
  }

  HRESULT GetHDOP(_Out_ double& dHDOP)
  {
    return GetValue(SENSOR_DATA_TYPE_HORIZONAL_DILUTION_OF_PRECISION, dHDOP);
  }

  HRESULT GetLatitude(_Out_ double& dLatitude)
  {
    return GetValue(SENSOR_DATA_TYPE_LATITUDE_DEGREES, dLatitude);
  }

  HRESULT GetLongitude(_Out_ double& dLongitude)
  {
    return GetValue(SENSOR_DATA_TYPE_LONGITUDE_DEGREES, dLongitude);
  }

  HRESULT GetMagneticHeading(_Out_ double& dHeading)
  {
    return GetValue(SENSOR_DATA_TYPE_MAGNETIC_HEADING_DEGREES, dHeading);
  }

  HRESULT GetMagenticVariation(_Out_ double& dVariation)
  {
    return GetValue(SENSOR_DATA_TYPE_MAGNETIC_VARIATION, dVariation);
  }

  HRESULT GetNMEASentence(_Out_ std::wstring& sSentence)
  {
    return GetValue(SENSOR_DATA_TYPE_NMEA_SENTENCE, sSentence);
  }

  HRESULT GetPDOP(_Out_ double& dVDOP)
  {
    return GetValue(SENSOR_DATA_TYPE_POSITION_DILUTION_OF_PRECISION, dVDOP);
  }

  HRESULT GetSatellitesInView(_Out_ int& nSatellites)
  {
    return GetValue(SENSOR_DATA_TYPE_SATELLITES_IN_VIEW, nSatellites);
  }

  HRESULT GetSatellitesInViewAzimuth(_Out_ std::vector<double>& dblArray)
  {
    return GetValue(SENSOR_DATA_TYPE_SATELLITES_IN_VIEW_AZIMUTH, dblArray);
  }

  HRESULT GetSatellitesInViewElevation(_Out_ std::vector<double>& dblArray)
  {
    return GetValue(SENSOR_DATA_TYPE_SATELLITES_IN_VIEW_ELEVATION, dblArray);
  }

  HRESULT GetSatellitesInViewIDs(std::vector<ULONG>& ulArray)
  {
    return GetValue(SENSOR_DATA_TYPE_SATELLITES_IN_VIEW_ID, ulArray);
  }

  HRESULT GetSatellitesUsed(_Out_ int& nSatellites)
  {
    return GetValue(SENSOR_DATA_TYPE_SATELLITES_USED_COUNT, nSatellites);
  }

  HRESULT GetSatellitesUsedPRNS(_Out_ std::vector<ULONG>& ulArray)
  {
    return GetValue(SENSOR_DATA_TYPE_SATELLITES_USED_PRNS, ulArray);
  }

  HRESULT GetSatellitesInViewSTNRatio(_Out_ std::vector<double>& dblArray)
  {
    return GetValue(SENSOR_DATA_TYPE_SATELLITES_IN_VIEW_STN_RATIO, dblArray);
  }

  HRESULT GetSpeed(_Out_ double& dSpeed)
  {
    return GetValue(SENSOR_DATA_TYPE_SPEED_KNOTS, dSpeed);
  }

  HRESULT GetStateProvince(_Out_ std::wstring& sStateProvince)
  {
    return GetValue(SENSOR_DATA_TYPE_STATE_PROVINCE, sStateProvince);
  }

  HRESULT GetTrueHeading(_Out_ double& dHeading)
  {
    return GetValue(SENSOR_DATA_TYPE_TRUE_HEADING_DEGREES, dHeading);
  }

  HRESULT GetVDOP(_Out_ double& dVDOP)
  {
    return GetValue(SENSOR_DATA_TYPE_VERTICAL_DILUTION_OF_PRECISION, dVDOP);
  }
};

//An Electrical sensor specific wrapper for ISensorDataReport
class CElectricalSensorDataReport : public CSensorDataReport
{
public:
//Constructors / Destructors
  CElectricalSensorDataReport(_In_ ISensorDataReport* pSensorDataReport) noexcept : CSensorDataReport{pSensorDataReport}
  {
  }
  CElectricalSensorDataReport(const CElectricalSensorDataReport&) = default;
  CElectricalSensorDataReport(CElectricalSensorDataReport&&) = delete;
  CElectricalSensorDataReport() = default;
  ~CElectricalSensorDataReport() = default;

//Methods
  CElectricalSensorDataReport& operator=(const CElectricalSensorDataReport&) = default;
  CElectricalSensorDataReport& operator=(CElectricalSensorDataReport&&) = delete;
  HRESULT GetCapacitance(_Out_ double& dCapacitance)
  {
    return GetValue(SENSOR_DATA_TYPE_CAPACITANCE_FARAD, dCapacitance);
  }

  HRESULT GetCurrent(_Out_ double& dCurrent)
  {
    return GetValue(SENSOR_DATA_TYPE_CURRENT_AMPS, dCurrent);
  }

  HRESULT GetPower(_Out_ double& dPower)
  {
    return GetValue(SENSOR_DATA_TYPE_ELECTRICAL_POWER_WATTS, dPower);
  }

  HRESULT GetInductance(_Out_ double& dInductance)
  {
    return GetValue(SENSOR_DATA_TYPE_INDUCTANCE_HENRY, dInductance);
  }

  HRESULT GetResistance(_Out_ double& dResistance)
  {
    return GetValue(SENSOR_DATA_TYPE_RESISTANCE_OHMS, dResistance);
  }

  HRESULT GetVoltage(_Out_ double& dVoltage)
  {
    return GetValue(SENSOR_DATA_TYPE_VOLTAGE_VOLTS, dVoltage);
  }
};

//A Environmental sensor specific wrapper for ISensorDataReport
class CEnvironmentalSensorDataReport : public CSensorDataReport
{
public:
//Constructors / Destructors
  CEnvironmentalSensorDataReport(_In_ ISensorDataReport* pSensorDataReport) noexcept : CSensorDataReport{pSensorDataReport}
  {
  }
  CEnvironmentalSensorDataReport(const CEnvironmentalSensorDataReport&) = default;
  CEnvironmentalSensorDataReport(CEnvironmentalSensorDataReport&&) = delete;
  CEnvironmentalSensorDataReport() = default;
  ~CEnvironmentalSensorDataReport() = default;

//Methods
  CEnvironmentalSensorDataReport& operator=(const CEnvironmentalSensorDataReport&) = default;
  CEnvironmentalSensorDataReport& operator=(CEnvironmentalSensorDataReport&&) = delete;
  HRESULT GetAtmosphericPressure(_Out_ float& fPressure)
  {
    return GetValue(SENSOR_DATA_TYPE_ATMOSPHERIC_PRESSURE_BAR, fPressure);
  }

  HRESULT GetTemperature(_Out_ float& fTemperature)
  {
    return GetValue(SENSOR_DATA_TYPE_TEMPERATURE_CELSIUS, fTemperature);
  }

  HRESULT GetRelativeHumidity(_Out_ float& fRelativeHumidity)
  {
    return GetValue(SENSOR_DATA_TYPE_RELATIVE_HUMIDITY_PERCENT, fRelativeHumidity);
  }

  HRESULT GetWindDirection(_Out_ float& fDirection)
  {
    return GetValue(SENSOR_DATA_TYPE_WIND_DIRECTION_DEGREES_ANTICLOCKWISE, fDirection);
  }

  HRESULT GetWindSpeed(_Out_ float& fSpeed)
  {
    return GetValue(SENSOR_DATA_TYPE_WIND_SPEED_METERS_PER_SECOND, fSpeed);
  }
};

//A Light sensor specific wrapper for ISensorDataReport
class CLightSensorDataReport : public CSensorDataReport
{
public:
//Constructors / Destructors
  CLightSensorDataReport(_In_ ISensorDataReport* pSensorDataReport) noexcept : CSensorDataReport{pSensorDataReport}
  {
  }
  CLightSensorDataReport(const CLightSensorDataReport&) = default;
  CLightSensorDataReport(CLightSensorDataReport&&) = delete;
  CLightSensorDataReport() = default;
  ~CLightSensorDataReport() = default;

//Methods
  CLightSensorDataReport& operator=(const CLightSensorDataReport&) = default;
  CLightSensorDataReport& operator=(CLightSensorDataReport&&) = delete;
  HRESULT GetChromacity(_Out_ std::vector<float>& fltArray)
  {
    return GetValue(SENSOR_DATA_TYPE_LIGHT_CHROMACITY, fltArray);
  }

  HRESULT GetLevel(_Out_ float& fLevel)
  {
    return GetValue(SENSOR_DATA_TYPE_LIGHT_LEVEL_LUX, fLevel);
  }

  HRESULT GetTemperature(_Out_ float& fTemperature)
  {
    return GetValue(SENSOR_DATA_TYPE_LIGHT_TEMPERATURE_KELVIN, fTemperature);
  }
};

//A Mechanical sensor specific wrapper for ISensorDataReport
class CMechanicalSensorDataReport : public CSensorDataReport
{
public:
//Constructors / Destructors
  CMechanicalSensorDataReport(_In_ ISensorDataReport* pSensorDataReport) noexcept : CSensorDataReport{pSensorDataReport}
  {
  }
  CMechanicalSensorDataReport(const CMechanicalSensorDataReport&) = default;
  CMechanicalSensorDataReport(CMechanicalSensorDataReport&&) = delete;
  CMechanicalSensorDataReport() = default;
  ~CMechanicalSensorDataReport() = default;

//Methods
  CMechanicalSensorDataReport& operator=(const CMechanicalSensorDataReport&) = default;
  CMechanicalSensorDataReport& operator=(CMechanicalSensorDataReport&&) = delete;
  HRESULT GetAbsolutePressure(_Out_ double& dAbsolutePressure)
  {
    return GetValue(SENSOR_DATA_TYPE_ABSOLUTE_PRESSURE_PASCAL, dAbsolutePressure);
  }

  HRESULT GetSwitchState(_Out_ bool& bState)
  {
    return GetValue(SENSOR_DATA_TYPE_BOOLEAN_SWITCH_STATE, bState);
  }

  HRESULT GetForce(_Out_ double& dForce)
  {
    return GetValue(SENSOR_DATA_TYPE_FORCE_NEWTONS, dForce);
  }

  HRESULT GetGaugePressure(_Out_ double& dGaugePressure)
  {
    return GetValue(SENSOR_DATA_TYPE_GAUGE_PRESSURE_PASCAL, dGaugePressure);
  }

  HRESULT GetMutiValueSwitchState(_Out_ double& dSwitchState)
  {
    return GetValue(SENSOR_DATA_TYPE_MULTIVALUE_SWITCH_STATE, dSwitchState);
  }

  HRESULT GetStrain(_Out_ double& dStrain)
  {
    return GetValue(SENSOR_DATA_TYPE_STRAIN, dStrain);
  }

  HRESULT GetWeight(_Out_ double& dWeight)
  {
    return GetValue(SENSOR_DATA_TYPE_WEIGHT_KILOGRAMS, dWeight);
  }
};

//A Motion sensor specific wrapper for ISensorDataReport
class CMotionSensorDataReport : public CSensorDataReport
{
public:
//Constructors / Destructors
  CMotionSensorDataReport(_In_ ISensorDataReport* pSensorDataReport) noexcept : CSensorDataReport{pSensorDataReport}
  {
  }
  CMotionSensorDataReport(const CMotionSensorDataReport&) = default;
  CMotionSensorDataReport(CMotionSensorDataReport&&) = delete;
  CMotionSensorDataReport() = default;
  ~CMotionSensorDataReport() = default;

//Methods
  CMotionSensorDataReport& operator=(const CMotionSensorDataReport&) = default;
  CMotionSensorDataReport& operator=(CMotionSensorDataReport&&) = delete;
  HRESULT GetAccelerationX(_Out_ double& dAccelerationX)
  {
    return GetValue(SENSOR_DATA_TYPE_ACCELERATION_X_G, dAccelerationX);
  }

  HRESULT GetAccelerationY(_Out_ double& dAccelerationY)
  {
    return GetValue(SENSOR_DATA_TYPE_ACCELERATION_Y_G, dAccelerationY);
  }

  HRESULT GetAccelerationZ(_Out_ double& dAccelerationZ)
  {
    return GetValue(SENSOR_DATA_TYPE_ACCELERATION_Z_G, dAccelerationZ);
  }

  HRESULT GetGyrometricAccelerationX(_Out_ double& dGyrometricAccelerationX)
  {
    return GetValue(SENSOR_DATA_TYPE_ANGULAR_ACCELERATION_X_DEGREES_PER_SECOND_SQUARED, dGyrometricAccelerationX);
  }

  HRESULT GetGyrometricAccelerationY(_Out_ double& dGyrometricAccelerationY)
  {
    return GetValue(SENSOR_DATA_TYPE_ANGULAR_ACCELERATION_Y_DEGREES_PER_SECOND_SQUARED, dGyrometricAccelerationY);
  }

  HRESULT GetGyrometricAccelerationZ(_Out_ double& dGyrometricAccelerationZ)
  {
    return GetValue(SENSOR_DATA_TYPE_ANGULAR_ACCELERATION_Z_DEGREES_PER_SECOND_SQUARED, dGyrometricAccelerationZ);
  }

  HRESULT GetState(_Out_ bool& bState)
  {
    return GetValue(SENSOR_DATA_TYPE_MOTION_STATE, bState);
  }

  HRESULT GetSpeed(_Out_ double& dSpeed)
  {
    return GetValue(SENSOR_DATA_TYPE_SPEED_METERS_PER_SECOND, dSpeed);
  }
};

//A Orientation sensor specific wrapper for ISensorDataReport
class COrientationSensorDataReport : public CSensorDataReport
{
public:
//Constructors / Destructors
  COrientationSensorDataReport(_In_ ISensorDataReport* pSensorDataReport) noexcept : CSensorDataReport{pSensorDataReport}
  {
  }
  COrientationSensorDataReport(const COrientationSensorDataReport&) = default;
  COrientationSensorDataReport(COrientationSensorDataReport&&) = delete;
  COrientationSensorDataReport() = default;
  ~COrientationSensorDataReport() = default;

//Methods
  COrientationSensorDataReport& operator=(const COrientationSensorDataReport&) = default;
  COrientationSensorDataReport& operator=(COrientationSensorDataReport&&) = delete;
  HRESULT GetTiltX(_Out_ float& fTiltX)
  {
    return GetValue(SENSOR_DATA_TYPE_TILT_X_DEGREES, fTiltX);
  }

  HRESULT GetTiltY(_Out_ float& fTiltY)
  {
    return GetValue(SENSOR_DATA_TYPE_TILT_Y_DEGREES, fTiltY);
  }

  HRESULT GetTiltZ(_Out_ float& fTiltZ)
  {
    return GetValue(SENSOR_DATA_TYPE_TILT_Z_DEGREES, fTiltZ);
  }

  HRESULT GetDistanceX(_Out_ float& fDistanceX)
  {
    return GetValue(SENSOR_DATA_TYPE_DISTANCE_X_METERS, fDistanceX);
  }

  HRESULT GetDistanceY(_Out_ float& fDistanceY)
  {
    return GetValue(SENSOR_DATA_TYPE_DISTANCE_Y_METERS, fDistanceY);
  }

  HRESULT GetDistanceZ(_Out_ float& fDistanceZ)
  {
    return GetValue(SENSOR_DATA_TYPE_DISTANCE_Z_METERS, fDistanceZ);
  }

  HRESULT GetMagneticHeadingX(_Out_ float& fMagenticHeadingX)
  {
    return GetValue(SENSOR_DATA_TYPE_MAGNETIC_HEADING_X_DEGREES, fMagenticHeadingX);
  }

  HRESULT GetMagneticHeadingY(_Out_ float& fMagenticHeadingY)
  {
    return GetValue(SENSOR_DATA_TYPE_MAGNETIC_HEADING_Y_DEGREES, fMagenticHeadingY);
  }

  HRESULT GetMagneticHeadingZ(_Out_ float& fMagenticHeadingZ)
  {
    return GetValue(SENSOR_DATA_TYPE_MAGNETIC_HEADING_Z_DEGREES, fMagenticHeadingZ);
  }
};

//A Scanner sensor specific wrapper for ISensorDataReport
class CScannerSensorDataReport : public CSensorDataReport
{
public:
//Constructors / Destructors
  CScannerSensorDataReport(_In_ ISensorDataReport* pSensorDataReport) noexcept : CSensorDataReport{pSensorDataReport}
  {
  }
  CScannerSensorDataReport(const CScannerSensorDataReport&) = default;
  CScannerSensorDataReport(CScannerSensorDataReport&&) = delete;
  CScannerSensorDataReport() = default;
  ~CScannerSensorDataReport() = default;

//Methods
  CScannerSensorDataReport& operator=(const CScannerSensorDataReport&) = default;
  CScannerSensorDataReport& operator=(CScannerSensorDataReport&&) = delete;
  HRESULT GetRFIDTag(_Out_ ULARGE_INTEGER& uhVal)
  {
    return GetValue(SENSOR_DATA_TYPE_RFID_TAG_40_BIT, uhVal);
  }
};

}; //namespace SensorAPI

#endif //#ifndef __MFCSENSOR_H__
